
dpp.pages.search = {

	title: 'menu-search',
	lastSearchResults: null,

	start: function()
	{
		ga('send', 'event', 'General', 'ViewSearch');

		dpp.pages.search.formReset();
		dpp.pages.search.displayMode('default');

		dpp.pages.search.loadLastRouteSearches();

		$('#search-accessible').tooltip({title: dpp.t('search-accessible-tooltip')});
		$('#search-lowfloor').tooltip({title: dpp.t('search-lowfloor-tooltip')});
		$('#search-transfers-button').tooltip({title: dpp.t('search-transfers-tooltip')});
	},

	adjustUi: function()
	{
		$('.form-row').each(function(index, el)
		{
			var row = $(el);
			var availableWidth = $(el).innerWidth();

			row.children('.layout-fixed').each(function(index, fixed)
			{
				availableWidth -= Math.floor($(fixed).width());
			})
			.promise()
			.done(function()
			{
				var autos = 0;
				row.children('.layout-auto').each(function()
				{
					autos++;
				})
				.promise()
				.done(function()
				{
					row.children('.layout-auto').each(function(index, auto)
					{
						auto = $(auto);
						auto.width(Math.floor(availableWidth / autos) - 5);
					});
				});
			});
		});
	},

	displayMode: function(mode)
	{
		console.log('Search display mode: ', mode);

		var recentRoutesEl = $('#search-recentroutes');

		switch (mode)
		{
			case 'input':
				if (!recentRoutesEl.is(':empty')) recentRoutesEl.collapse('show');
				$('#search-results').collapse('hide');
				$('#search-info').collapse('hide');
				$('#search-showinfo').show();
				$('.search-form-collapse').show();
			break;
			case 'search':
				recentRoutesEl.collapse('hide');
				$('#search-results').collapse('show');
				$('#search-info').collapse('hide');
				$('#search-showinfo').show();
				$('.search-form-collapse').show();
			break;
			default:
				recentRoutesEl.collapse('hide');
				$('#search-results').collapse('hide');
				$('#search-info').collapse('show');
				$('#search-showinfo').hide();
				$('.search-form-collapse').hide();
			break;
		}

		dpp.pages.search.adjustUi();
	},

	formReset: function()
	{
		$('#search-time').val((new Date()).time24());

		switch (dpp.getLang())
		{
			case 'cs':
				$('#search-date').val(Date.today().formatCs());
			break;
			default:
				$('#search-date').val(Date.today().formatInt());
			break;
		}
	},

	formToggleDepartureArrival: function()
	{
		var el = $('#search-time-da');

		if (el.attr('data-t-id') == 'search-departure')
		{
			el.html(dpp.t('search-arrival'));
			el.attr('data-t-id', 'search-arrival');
		}
		else
		{
			el.html(dpp.t('search-departure'));
			el.attr('data-t-id', 'search-departure');
		}
	},

	formIsDeparture: function()
	{
		if ($('#search-time-da').attr('data-t-id') == 'search-departure')
			return true;
		return false;
	},

	formMaxTransfers: function()
	{
		return parseInt($('#search-transfers-button-number').html());
	},

	formSelectMaxTransfers: function(transfers)
	{
		$('#search-transfers-button-number').html(transfers)
	},

	formToggleAccessible: function()
	{
		if ($('#search-accessible').hasClass('active'))
			$('#search-accessible').removeClass('active');
		else
			$('#search-accessible').addClass('active');
	},

	formToggleLowFloor: function()
	{
		if ($('#search-lowfloor').hasClass('active'))
			$('#search-lowfloor').removeClass('active');
		else
			$('#search-lowfloor').addClass('active');
	},

	formIsAccessible: function()
	{
		if ($('#search-accessible').hasClass('active'))
			return true
		return false;
	},

	formIsLowFloor: function()
	{
		if ($('#search-lowfloor').hasClass('active'))
			return true
		return false;
	},
		
	loadLastRouteSearches: function()
	{
		var lastRoutes = window.localStorage.getItem('lastRoutes');
		var count = parseInt(dpp.getStoredValue('settings-lastRoutesCount', 5));
		
		if ((lastRoutes === undefined) || (lastRoutes === 'undefined'))
			return;
		
		lastRoutes = JSON.parse(lastRoutes);

		var list = '';
		for (var i in lastRoutes)
		{
			if (count > 0)
				list += '<button type="button" class="btn btn-outline-secondary btn-sm button-route" onclick="dpp.pages.search.fillRoute(\''+lastRoutes[i].start+'\', \''+lastRoutes[i].finish+'\');">'+lastRoutes[i].start+' - '+lastRoutes[i].finish+'</button>';

			count--;
		}
		
		var divLastSearches = $('#search-recentroutes');
		divLastSearches.html(list);
	},
	
	saveLastRouteSearch: function(sta, fin)
	{
		var saveCount = parseInt(dpp.getStoredValue('settings-lastRoutesCount', 5));
		
		if (saveCount == 0)
		{
			console.log('Not saving route, because it is not wanted.');
			$('#search-recentroutes').html('');
			return;
		}
		
		var lastRoutes = window.localStorage.getItem('lastRoutes');
		var newLastRoutes = [{start: sta, finish: fin}];

		if ((lastRoutes !== undefined) && (lastRoutes !== 'undefined'))
		{
			lastRoutes = JSON.parse(lastRoutes);
			saveCount--;
				
			for (var i in lastRoutes)
			{
				if (saveCount > 0)
				{
					if ((lastRoutes[i].start+lastRoutes[i].finish) != (sta+fin))
						newLastRoutes.push({start: lastRoutes[i].start, finish: lastRoutes[i].finish});
						
					saveCount--;
				}
			}
		}
		
		window.localStorage.setItem('lastRoutes', JSON.stringify(newLastRoutes));
		dpp.pages.search.loadLastRouteSearches();
	},
	
	fillRoute: function(start, finish)
	{
        $('#search-start').val(start);
        $('#search-destination').val(finish);
	},
	
	getRecentStops: function()
	{
		var saveCount = parseInt(dpp.getStoredValue('settings-recentStopsCount', 5));
		var recentStops = window.localStorage.getItem('recentStops');
		
		if (saveCount == 0)
			return;
		
		if (recentStops)
			return JSON.parse(recentStops);
		else
			return [];
	},
	
	saveStations: function(start, finish, across)
	{
		var saveCount = parseInt(dpp.getStoredValue('settings-recentStopsCount', 5));
		
		if (saveCount == 0)
			return;

		var recentStops = window.localStorage.getItem('recentStops');
		recentStops = JSON.parse(recentStops);

		if (across)
			var newrecentStops = [finish, start, across];
		else
			var newrecentStops = [finish, start];

		if (recentStops)
		{
			if (across)
				if (recentStops.indexOf(across) >= 0)
					recentStops.splice(recentStops.indexOf(across), 1);

			if (recentStops.indexOf(start) >= 0)
				recentStops.splice(recentStops.indexOf(start), 1);

			if (recentStops.indexOf(finish) >= 0)
				recentStops.splice(recentStops.indexOf(finish), 1);
		}
		else
			recentStops = [];

		newrecentStops = newrecentStops.concat(recentStops);

		newrecentStopsTrimmed = [];

		for (var i = 0; i < saveCount; i++)
			if (newrecentStops[i])
				newrecentStopsTrimmed.push(newrecentStops[i]);
		
		window.localStorage.setItem('recentStops', JSON.stringify(newrecentStopsTrimmed));
	},
		
	findRoutes: function(address)
    {
		if (address === undefined)
		{
			var whereFrom = $('#search-start').val();
			var whereTo = $('#search-destination').val();
			var whereAcross = $('#search-via').val();
			//var date = $('#search-date').val();
			var date = $('input[name="search-date-datepicker"]').val();
			var time = $('#search-time').val();
			
			var isDeparture = '1';
			if (!dpp.pages.search.formIsDeparture())
				isDeparture = '0';
			
			var accessibleOrLowerFloor = '';
			if (dpp.pages.search.formIsAccessible())
				accessibleOrLowerFloor = 'ldc=1&';
			if (dpp.pages.search.formIsLowFloor())
				accessibleOrLowerFloor = 'ldct=1&';
			
			var transfers = dpp.pages.search.formMaxTransfers();

			var doTransfer = '0';
			if (transfers)
				doTransfer = '1';
			
        
			//console.log('Searching route between '+whereFrom+' and '+whereTo+' across '+whereAcross+' at '+time+' ('+date+'). Is departure time: '+isDeparture+'; LowerFloor/Accessible'+accessibleOrLowerFloor+'; DoTransfer:'+doTransfer+'; Transfers:'+transfers+';');
  
			address = dpp.safeUrl('http://spojeni.dpp.cz/ConnForm.aspx?cl=C&res=1&tom=0&f='+whereFrom+'&t='+whereTo+'&v='+whereAcross+'&date='+date+'&time='+time+'&isdep='+isDeparture+'&alg='+doTransfer+'&chn='+transfers+'&'+accessibleOrLowerFloor+'cmdSearch=vyhledat');
		}

		dpp.showLoading();
		$.ajax({
			type: 'GET',
			url: address,
			dataType: 'html',
			error: function(response, status, msg)
			{
				dpp.displayError(dpp.t('search-nointernet'));
			},
			success: function(data)
			{
				data = $(data);
				var routesDiv = $('#search-results');

				if (data.find('div.spojeni').length == 0)
				{
					dpp.displayInfo(dpp.t('search-noconnections'));
					return;
				}

				var paging = $(data).find('#ctlPaging_ctlPaging')[0].getElementsByTagName('a');
				var routesBefore = '';
				var routesAfter = '';

				if (paging[0] !== undefined)
					if (paging[0].innerHTML == 'předchozí')
						routesBefore = '<button type="button" class="btn btn-info btn-sm" onclick="dpp.pages.search.findRoutes(dpp.safeUrl(\'http://spojeni.dpp.cz/'+paging[0].href.replace('file:///', '').replace('C:/', '').replace('http://dppapp.cz/', '')+'\'));"><i class="fa fa-arrow-circle-o-left"></i> '+dpp.t('search-previous')+'</button>';
					else
						routesAfter = '<button type="button" class="btn btn-info btn-sm pull-right" onclick="dpp.pages.search.findRoutes(dpp.safeUrl(\'http://spojeni.dpp.cz/'+paging[0].href.replace('file:///', '').replace('C:/', '').replace('http://dppapp.cz/', '')+'\'));"><i class="fa fa-arrow-circle-o-right"></i> '+dpp.t('search-next')+'</button>';

				if (paging[1] !== undefined)
					if (paging[1].innerHTML == 'předchozí')
						routesBefore = '<button type="button" class="btn btn-info btn-sm" onclick="dpp.pages.search.findRoutes(dpp.safeUrl(\'http://spojeni.dpp.cz/'+paging[1].href.replace('file:///', '').replace('C:/', '').replace('http://dppapp.cz/', '')+'\'));"><i class="fa fa-arrow-circle-o-left"></i> '+dpp.t('search-previous')+'</button>';
					else
						routesAfter = '<button type="button" class="btn btn-info btn-sm pull-right" onclick="dpp.pages.search.findRoutes(dpp.safeUrl(\'http://spojeni.dpp.cz/'+paging[1].href.replace('file:///', '').replace('C:/', '').replace('http://dppapp.cz/', '')+'\'));"><i class="fa fa-arrow-circle-o-right"></i> '+dpp.t('search-next')+'</button>';
				

				data = dpp.pages.search.parseRoutes(data);
				dpp.pages.search.lastSearchResults = data;
				
				routesDiv.html(dpp.pages.search.formatRoutes(data));
				routesDiv.append('<div id="paging">'+routesBefore+routesAfter+'<div>');

				dpp.pages.search.displayMode('search');
			},
			complete: function()
			{
				dpp.hideLoading();
			}
		});
	},
	
	parseRoutes: function(html)
	{
		html = $(html);
		
		var routes = [];
		var tmp;
		var route;
		html.find('div.spojeni').each(function (i, el)
		{
			var rootClasses = $(el).attr('class').split(' ');

			var idClass;
			for (var i in rootClasses)
				if (rootClasses[i].indexOf('_') >= 0)
					idClass = rootClasses[i];

			var departure = $(html.find('h2.'+idClass)[0]).children('span.date')[0].innerHTML.split(' ');
			var departureDate = departure[2]+'-'+departure[1].replace('.', '')+'-'+departure[0].replace('.', '');

			route = {};
			
			// parse departure time, arrival time, number of transfers
			tmp = el.getElementsByTagName('h3')[0].innerText.replace('až', ',').split(',');
			
			route['departure'] = tmp[0].trim();
			route['arrival'] = tmp[1].trim();
			route['departureDate'] = departureDate;
			
			if (tmp.length > 3)
			{
				route['transfers'] = tmp[2].trim();
				route['travelTime'] = tmp[3].trim();
			}
			else
			{
				route['transfers'] = false;
				route['travelTime'] = tmp[2].trim();
			}
			
			var paragraphs = el.getElementsByTagName('p');
			
			route['steps'] = [];
			for (var i in paragraphs)
			{
				// parse the ticket price
				if (paragraphs[i].className == 'note')
					route['price'] = paragraphs[i].innerText.substring(6, 9);
			
				// parse route steps
				if (paragraphs[i].className == 'walk')
					route['steps'].push({info: paragraphs[i].innerText.trim().replace(' . .', '.'), type: 'walk'});
				
				if (paragraphs[i].className == 'usek')
				{
					var step = paragraphs[i].getElementsByTagName('span');
					var info = {};
					var signs = [];
					for (var n in step)
					{
						if (step[n].className == 'start')
						{
							info['start'] = {station: step[n].getElementsByTagName('a')};
							
							if (info['start'].station.length > 0)
								info['start'].station = step[n].getElementsByTagName('a')[0].innerText;
							else
								info['start'].station = step[n].childNodes[1].nodeValue.trim();
							
							signs = step[n].getElementsByTagName('img');
							
							info['start']['time'] = step[n].innerText.substring(step[n].innerText.length - 5, step[n].innerText.length);
							
							info['start']['signs'] = [];
							var sign = '';
							for (var i in signs)
							{
								sign = signs[i].src;
								
								if (sign !== undefined)
								{
									sign = sign.split('/');
									info['start']['signs'].push(sign[sign.length - 1]);
								}
							}
						}
						
						if (step[n].className == 'cil')
						{
							
							info['finish'] = {station: step[n].getElementsByTagName('a')};
							
							if (info['finish'].station.length > 0)
								info['finish'].station = step[n].getElementsByTagName('a')[0].innerText;
							else
								info['finish'].station = step[n].childNodes[1].nodeValue.trim();
							
							signs = step[n].getElementsByTagName('img');
							
							info['finish']['time'] = step[n].innerText.substring(step[n].innerText.length - 8, step[n].innerText.length - 3);
							
							info['finish']['signs'] = [];
							var sign = '';
							for (var i in signs)
							{
								sign = signs[i].src;
								
								if (sign != undefined)
								{
									sign = sign.split('/');
									info['finish']['signs'].push(sign[sign.length - 1]);
								}
							}
						}
						
						if (
							(step[n].className != 'sipka') && 
							(step[n].className != 'start') && 
							(step[n].className != 'cil')
						)
						{
							if (step[n]['getElementsByTagName'])
							{
								var imgs = step[n].getElementsByTagName('img');
								if (imgs.length > 0)
								{
									var imgPath = imgs[0].src.split('/');
									info['driveSign'] = imgPath[imgPath.length - 1];
									
									info['driveLine'] = step[n].getElementsByTagName('span')[1].innerText;
								}
							}
						}
					}
					
					route['steps'].push({info: info, type: 'drive'});
				}
			}
			
			routes.push(route);
		});
		
		return routes;
	},
	
	formatRoutes: function(routes)
	{
		var list = '';
		
		for (var route in routes)
		{
			if (dpp.getLang() != 'cs')
			{
				routes[route].transfers = routes[route].transfers.replace('přestup', dpp.t('search-transfer'));
				routes[route].transfers = routes[route].transfers.replace('přestupy', dpp.t('search-transfers'));
			}

			routes[route].travelTime = routes[route].travelTime.replace('hod', dpp.t('search-hour-short'));
			routes[route].travelTime = routes[route].travelTime.replace('min.', dpp.t('search-minute-short'));
			routes[route].travelTime = routes[route].travelTime.replace('min', dpp.t('search-minute-short'));

			list += '<div class="card route">';
			list += '<div class="card-header"><b>'+routes[route].departure+' - '+routes[route].arrival+', '+routes[route].travelTime+'., '+routes[route].price+' '+(dpp.getLang()!='cs'?'CZK':'Kč')+', '+routes[route].transfers+'</b><button type="button" class="btn btn-outline-secondary btn-sm pull-right save" onclick="dpp.pages.savedconnections.saveConnection(this, \''+route+'\');"><i class="fa fa-save"></i></button></div>';
			list += '<div class="card-block">';
			
			list += '<ol>';
			var step;
			for (var stepIterator in routes[route].steps)
			{
				step = routes[route].steps[stepIterator];
				
				if (step.type == 'drive')
				{
					// start station
					list += '<li><b>'+step.info.start.station+'</b> ';
					for (var sign in step.info.start.signs)
						list += '<img src="img/FC/'+step.info.start.signs[sign]+'" alt="">';
					list += ' '+step.info.start.time+'</b> ';
					
					// line
					var color;
					switch (step.info.driveLine)
					{
						case 'A': color = 'green'; break;
						case 'B': color = '#fec831'; break;
						case 'C': color = 'red'; break;
						default: color = 'inherit'; break;
					}
					
					list += ' &rarr; ';
					list += '<img src="img/'+step.info.driveSign+'" alt=""> <b style="color: '+color+';">'+step.info.driveLine+'</b>';
					list += ' &rarr; ';
					
					// end station
					list += '<b>'+step.info.finish.station+'</b> ';
					for (var sign in step.info.finish.signs)
						list += '<img src="img/FC/'+step.info.finish.signs[sign]+'" alt="">';
					list += ' '+step.info.finish.time+'</b></li>';
				}
				
				if (step.type == 'walk')
				{
					if (dpp.getLang() != 'cs')
					{
						step.info = step.info.replace('Přesun pěšky asi', 'Transfer by foot, about');
						step.info = step.info.replace('minuty', 'many');
						step.info = step.info.replace('minuta', 'one');
						step.info = step.info.replace('minut', 'many');

						step.info = step.info.replace('one', dpp.t('search-minute'));
						step.info = step.info.replace('many', dpp.t('search-minutes'));
					}

					list += '<span class="routes-walk">'+step.info+'</span>';
				}
			}
			list += '</ol>';
			
			list += '</div>';
			list += '</div>';
		}
		
		return list;
	},

	formCheck: function(otherWay)
	{
		var whereFrom = $('#search-start').val();
		var whereTo = $('#search-destination').val();
		var whereAcross = $('#search-via').val();

		if (otherWay)
		{
			$('#search-start').val(whereTo);
			$('#search-destination').val(whereFrom);
			whereFrom = $('#search-start').val();
			whereTo = $('#search-destination').val();
		}

		var message = null;
		
		if (whereFrom === '')
		{
			dpp.displayError(dpp.t('search-error1'));
			return;
		}
		
		if (whereTo == '')
		{
			dpp.displayError(dpp.t('search-error2'));
			return;
		}
		
		if (whereFrom == whereTo)
		{
			dpp.displayError(dpp.t('search-error3'));
			return;
		}
		
		if (whereFrom == whereAcross)
		{
			dpp.displayError(dpp.t('search-error4'));
			return;
		}
		
		if (whereTo == whereAcross)
		{
			dpp.displayError(dpp.t('search-error5'));
			return;
		}
		
		var time = $('#search-time').val().split(':');
		if (time.length == 2)
		{
			var hours = parseInt(time[0]);
			var minutes = parseInt(time[1]);
			
			if ((hours < 0) || (hours > 23) || (minutes < 0) || (minutes > 59) || isNaN(hours) || isNaN(minutes))
			{
			dpp.displayError(dpp.t('search-error6'));
				return;
			}
			
			/*if (String(hours).length < 2)
				hours = '0'+hours;*/
			
			if (String(minutes).length < 2)
				minutes = '0'+minutes;
			
			$('#search-time').val(hours+':'+minutes);
		}
		else
		{
			dpp.displayError(dpp.t('search-error6'));
			return;
		}
		
		/*var date = $('#search-date-datepicker').val().split('.');
		if (date.length == 3)
		{
			var day = parseInt(date[0]);
			var month = parseInt(date[1]);
			var year = parseInt(date[2]);
			var currentYear = (new Date()).getFullYear();
			
			if ((day < 1) || (day > 31) || (month < 1) || (month > 12) || (year < currentYear) || (year > currentYear+2) || !day || !month || !year)
			{
				dpp.displayError(dpp.t('search-error7'));
				return;
			}
		}
		else
		{
			dpp.displayError(dpp.t('search-error7'));
			return;
		}*/

		dpp.pages.search.saveLastRouteSearch(whereFrom, whereTo);
		dpp.pages.search.saveStations(whereFrom, whereTo, whereAcross);
		
		dpp.pages.search.findRoutes();
	},
}